package org.dromara.system.controller.system;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.ObjectUtil;
import com.qiniu.util.Auth;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.QueryGroup;
import org.dromara.common.oss.core.OssClient;
import org.dromara.common.oss.entity.UploadResult;
import org.dromara.common.oss.factory.OssFactory;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.mybatis.core.page.TableDataInfo;
import org.dromara.system.domain.bo.SysOssBo;
import org.dromara.system.domain.bo.SysOssConfigBo;
import org.dromara.system.domain.vo.SysOssConfigVo;
import org.dromara.system.domain.vo.SysOssFilesVo;
import org.dromara.system.domain.vo.SysOssUploadVo;
import org.dromara.system.domain.vo.SysOssVo;
import org.dromara.system.service.ISysOssConfigService;
import org.dromara.system.service.ISysOssService;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.NotEmpty;
import lombok.RequiredArgsConstructor;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

/**
 * 文件上传 控制层
 *
 * @author Lion Li
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/resource/oss")
public class SysOssController extends BaseController {

    private final ISysOssService ossService;

    private final ISysOssConfigService ossConfigService;

    /**
     * 查询OSS对象存储列表
     */
    @GetMapping("/list")
    public TableDataInfo<SysOssVo> list(@Validated(QueryGroup.class) SysOssBo bo, PageQuery pageQuery) {
        return ossService.queryPageList(bo, pageQuery);
    }

    @GetMapping("/allArchivesFileList")
    public TableDataInfo<SysOssFilesVo> listfile(@Validated(QueryGroup.class) SysOssBo bo, PageQuery pageQuery) {
        return ossService.queryPageFileList(bo,pageQuery);
    }

    /**
     * 查询OSS对象基于id串
     *
     * @param ossIds OSS对象ID串
     */
    @GetMapping("/listByIds/{ossIds}")
    @Log(title = "OSS对象存储", businessType = BusinessType.OTHER)
    public R<List<SysOssVo>> listByIds(@NotEmpty(message = "主键不能为空")
                                       @PathVariable Long[] ossIds,@RequestParam(required = false,defaultValue = "1") String type ) {

        List<SysOssVo> list = ossService.listByIds(Arrays.asList(ossIds));
        return R.ok(list);
    }

    @PutMapping("/update")
    public R update(SysOssBo bo) {
        if(bo.getOssId()!=null && bo.getSortindex()!=null){
            ossService.updateOssSortindex(bo.getOssId(),bo.getSortindex());
            return R.ok();
        }
        return R.ok();
    }

    /**
     * 上传OSS对象存储
     *
     * @param file 文件
     */
    @Log(title = "OSS对象存储", businessType = BusinessType.INSERT)
    @PostMapping(value = "/upload", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public R<SysOssUploadVo> upload(@RequestPart("file") MultipartFile file) {
        if (ObjectUtil.isNull(file)) {
            return R.fail("上传文件不能为空");
        }
        SysOssVo oss = ossService.upload(file);
        SysOssUploadVo uploadVo = new SysOssUploadVo();
        uploadVo.setUrl(oss.getUrl());
        uploadVo.setFileName(oss.getOriginalName());
        uploadVo.setOssId(oss.getOssId().toString());
        return R.ok(uploadVo);
    }

    @Log(title = "OSS对象存储", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    public R<SysOssUploadVo> addfile(@RequestBody SysOssBo bo) {
        UploadResult p = UploadResult.builder().build();
        p.setFilename(bo.getFileName());
        p.setUrl(bo.getUrl());
        SysOssUploadVo uploadVo = new SysOssUploadVo();
        SysOssVo oss = ossService.buildResultEntity(bo.getOriginalName(), bo.getFileSuffix(), bo.getSize(), "qiniu", p);
        uploadVo.setUrl(oss.getUrl());
        uploadVo.setFileName(oss.getOriginalName());
        uploadVo.setOssId(oss.getOssId().toString());
       return  R.ok(uploadVo);
    }

    /**
     * 下载OSS对象
     *
     * @param ossId OSS对象ID
     */
    @Log(title = "OSS对象存储", businessType = BusinessType.EXPORT)
    @GetMapping("/download/{ossIds}")
    public void download(@PathVariable String ossIds, HttpServletResponse response) throws IOException {
        if(ossIds.split(",").length>1){
            ossService.downloads(Arrays.stream(ossIds.split(",")).map(i->Long.parseLong(i)).toList(), response);
        }else{
            ossService.download(Long.parseLong(ossIds), response);
        }

    }

    /**
     * 删除OSS对象存储
     *
     * @param ossIds OSS对象ID串
     */
    @Log(title = "OSS对象存储", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ossIds}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ossIds) {
        return toAjax(ossService.deleteWithValidByIds(List.of(ossIds), true));
    }


    @GetMapping("/qiniu/token")
    public R<Object> gettoken(String url) {
        OssClient storage = OssFactory.instance("qiniu");
        SysOssConfigBo bo =new SysOssConfigBo();
        bo.setConfigKey("qiniu");
        TableDataInfo<SysOssConfigVo> pageList = ossConfigService.queryPageList(bo,new PageQuery());

        String upToken = "";
        if(pageList.getRows().size()>0){
            Auth auth = Auth.create(pageList.getRows().get(0).getAccessKey(),  pageList.getRows().get(0).getSecretKey());
            upToken = auth.uploadToken(pageList.getRows().get(0).getBucketName());

        }
        return R.ok((Object) upToken);


    }


}
